; NOTE: If you want to modify this code, you should reassemble it with SPASM.
; Other assemblers will not understand the macros that make dealing with funky
; Axe limitations easier.
.nolist

#include "ti83plus.inc"
#include "Axe.inc"
#include "TokenHook.inc"

#macro RESET_REORG_USAGE_COUNTER
	#ifndef USAGE_COUNTER
		.echo "REORG macros initalized."
		#define USAGE_COUNTER 0
	#else
		.echo "Previous REORG offset: ", USAGE_COUNTER
		#undefine USAGE_COUNTER
		#define USAGE_COUNTER 0
	#endif
#endmacro
#macro INCREMENT_USAGE_COUNTER
	#define NEW_USAGE_COUNTER eval(USAGE_COUNTER+1)
	#undefine USAGE_COUNTER
	#define USAGE_COUNTER eval(NEW_USAGE_COUNTER)
	#undefine NEW_USAGE_COUNTER
#endmacro
#macro REPLACE_NEXT
	.db $7F
	.org $-1
	INCREMENT_USAGE_COUNTER
#endmacro
#macro ABSOLUTE_NEXT
	.db $49
	.org $-1
	INCREMENT_USAGE_COUNTER
#endmacro
#ifdef b_jump
	#undefine b_jump
#endif
#macro b_jump(xxxx)
	ABSOLUTE_NEXT
	call 50h
	.dw xxxx
#endmacro

;#define DEBUG

#ifdef DEBUG
	#define BREAK or a \ jr nc, $
	#macro DISPHL
		push	af
		push	bc
		push	de
		push	hl
		b_call(_DispHL)
		b_call(_NewLine)
		pop	hl
		pop	de
		pop	bc
		pop	af
	#endmacro
	#macro DISPDE
		push	af
		push	bc
		push	de
		push	hl
		ex	de, hl
		b_call(_DispHL)
		b_call(_NewLine)
		pop	hl
		pop	de
		pop	bc
		pop	af
	#endmacro
	#macro DISPBC
		push	af
		push	bc
		push	de
		push	hl
		ld	l, c
		ld	h, b
		b_call(_DispHL)
		b_call(_NewLine)
		pop	hl
		pop	de
		pop	bc
		pop	af
	#endmacro
	#macro DispChar(x)
		push	af
		ld	a, x
		b_call(_PutC)
		pop	af
	#endmacro
#else
	#define BREAK .echo "Nope, no breakpoints."
	#define DISPHL .echo "Skipped DISPHL."
	#define DISPDE .echo "Skipped DISPDE."
	#define DISPBC .echo "Skipped DISPBC."
	#define DispChar(x) .echo "Skipped DispChar(", x, ")"
#endif
.list


;====== Equates ================================================================

prgmPage	.equ	8000h
prgmData	.equ	8001h
prgmSize	.equ	8003h
prgmCode	.equ	8005h


;====== Header =================================================================

	.dw	AXM_HEADER


;====== Routine 1: RunArchdPrgm( ===============================================

	.dw	routine1Size
	.db	AXM_ALL
	.db	t2ByteTok, t2PropZInt
	.db	AXM_SUB
	.db	AXM_1ARG
	.org	0
	RESET_REORG_USAGE_COUNTER
runArchivedProgram = sub_Axiom1

	rst	20h	; rMOV9TOOP1
	b_call(_ChkFindSym)
	;ret	c
	jr	nc, +_
	b_jump(_ErrUndefined)
_:	ex	de, hl
	ld	(prgmData), hl
	ld	a, b
	ld	(prgmPage), a
	or	a
	jr	nz, +_
	b_jump(_ErrDomain)
_:	; It's not in RAM, so we have to fiddle with the pointers to get its size
	ld	de, 9
	add	hl, de
	call	fixPointer
	b_call(_LoadCIndPaged)	; Get name size
	DISPBC
	DISPHL
	ld	e, c
	inc	e
	ld	d, 0
	add	hl, de
	call	fixPointer
	b_call(_LoadDEIndPaged)	; Size word
	DISPDE
	ld	(prgmSize), de
	;inc	hl		; _LoadDEIndPaged increments HL by one (not two)
	inc	hl
	call	fixPointer
	b_call(_LoadDEIndPaged)	; Compiled assembly program token header
	DISPDE
	ld	a, e
	cp	0BBh
	jr	nz, +_
	ld	a, d
	cp	6Dh
	jr	z, ++_
_:	b_jump(_ErrInvalid)
_:	;inc	hl
	inc	hl
	call	fixPointer
	ld	a, b
	ld	(prgmPage), a
	ld	(prgmData), hl
	; Now we have the size.  Next, we check if we have the RAM, too.
	ld	hl, (prgmSize)
	b_call(_EnoughMem)
	;ret	c	; No?  Give up.
	jr	nc, +_
	b_jump(_ErrMemory)
_:	; This is the semi-tricky part.  Since InsertMem and DelMem will move us
	; around if we're in a RAM program, we need to copy our code to some
	; scratch RAM
	DispChar('A')
	REPLACE_NEXT
	ld	hl, executionStub
	ld	de, prgmCode
	ld	bc, executionStubSize
	ldir
	DispChar('B')
	jp	prgmCode
fixPointer:
	ld	a, h
	cp	80h
	jr	c, +_
	sub	40h
	ld	h, a
	inc	b
_:	ret
executionStub:
	.org	prgmCode
executionStubStart:
	DispChar('C')
	ld	hl, (prgmSize)
	ld	de, 9D95h
	b_call(_InsertMem)
;	ld	hl, (asm_prgm_size)
;	push	hl
;	ld	hl, (prgmSize)
;	ld	(asm_prgm_size), hl
	ld	a, (prgmPage)
#ifdef DEBUG	
	ld	l, a
	ld	h, 0
#endif
	DISPHL
;	BREAK
	ld	hl, (prgmData)
	DISPHL
	ld	de, 9D95h
	DISPDE
	ld	bc, (prgmSize)
	DISPBC
	push	bc
	b_call(_FlashToRam)
	DispChar('D')
	; Why didn't I redefine AppOnErr() and AppOffErr?  I dunno.
	ld	hl, errRoutine
	ABSOLUTE_NEXT
	call	APP_PUSH_ERRORH
#ifdef SPECIAL_HEADER
	ld	hl, 9D95h		; 3
	ABSOLUTE_NEXT
	ld	de, ++_			; 3
	ld	b, +++_-++_		; 2
_:	ld	a, (de)			; 1
	cp	(hl)			; 1
	jr	nz, ++_			; 2
	inc	hl			; 1
	inc	de			; 1
	djnz	-_			; 2  Sigma=16
	DispChar('e')
	call	9D98h
	jr	+++_
_:	djnz	$	; 10 FE
	ret		; C9
#endif
_:	DispChar('E')
	call	9D95h
_:	ABSOLUTE_NEXT
	call	APP_POP_ERRORH
	xor	a
errRoutine:
	DispChar('F')
	pop	de
	ld	hl, 9D95h
	push	af
	b_call(_DelMem)
	pop	af
;	pop	hl
;	ld	(asm_prgm_size), hl
	or	a
	ret	z
	and	7Fh
	b_jump(_JError)
executionStubEnd:

executionStubSize = executionStubEnd - executionStubStart
routine1Size = executionStub + (executionStubSize) + USAGE_COUNTER
.echo "routine1size: ", routine1Size


;====== Routine 2: RunPrgm( ====================================================
	.dw	routine2Size
	.db	AXM_ALL
	.db	t2ByteTok, t1PropZInt
	.db	AXM_SUB
	.db	AXM_1ARG
	.org	0
	RESET_REORG_USAGE_COUNTER

	rst	20h	; rMOV9TOOP1
	b_call(_ChkFindSym)
	; If the program doesn't exist, RunRAMPrgm or RunArchdPrgm will catch
	; that.  Whichever one catches that doesn't matter, so the fact that B
	; may contain garbage doesn't matter.
;	jr	nc, +_
;	b_jump(_ErrUndefined)
_:	ld	a, b
  	or	a
	ld	hl, OP1
	jp	z, runRAMCore
	jp	runArchivedProgram

routine2Size = $ + USAGE_COUNTER


;====== Routine 3: RunRAMPrgm( =================================================

	.dw	routine3Size
	.db	AXM_ALL
	.db	t2ByteTok, tChiTest
	.db	AXM_SUB
	.db	AXM_1ARG
	.org	0
runRAMCore = sub_Axiom3
	RESET_REORG_USAGE_COUNTER

; You need to use an error handler here or else asm_prgm_size won't be restored
; if an error is thrown
	rst	20h	; rMOV9TOOP1
	; _ExecutePrgm will check whether or not the program exists for us.
	ld	hl, (asm_prgm_size)
	push	hl
	REPLACE_NEXT
	ld	hl, +_
	ABSOLUTE_NEXT
	call	APP_PUSH_ERRORH
	bcall(_ExecutePrgm)
	ABSOLUTE_NEXT
	call	APP_POP_ERRORH
	xor	a
_:	pop	hl
	ld	(asm_prgm_size), hl
	or	a
	ret	z
	and	7Fh
	b_jump(_JError)

routine3Size = $ + USAGE_COUNTER


;====== Routine 4: AppOnErr( ===================================================

	.dw	routine4Size
	.db	AXM_ALL
	.db	t2ByteTok, t2SampZInt
	.db	AXM_INLINE
	.db	AXM_1ARG
	.org	0
	RESET_REORG_USAGE_COUNTER

	ABSOLUTE_NEXT
	call	APP_PUSH_ERRORH

routine4Size = $ + USAGE_COUNTER


;====== Routine 5: AppOffErr ===================================================

	.dw	routine5Size
	.db	AXM_ALL
	.db	t2ByteTok, t2SampTInt
	.db	AXM_INLINE
	.db	AXM_0ARGS
	.org	0
	RESET_REORG_USAGE_COUNTER

	ABSOLUTE_NEXT
	call	APP_POP_ERRORH

routine5Size = $ + USAGE_COUNTER


;====== Routine 6: Copy A to HL ================================================

	.dw	routine6Size
	.db	AXM_ALL
	.db	t2ByteTok, t2SampFTest
	.db	AXM_INLINE
	.db	AXM_0ARGS
	.org	0

	and	7Fh
	ld	l, a
	ld	h, 0

routine6Size:


;====== Tokens =================================================================

	.dw	AXM_END

	.dw	hChiTest
_:	.db	+_--_-1
	.db	"RunRAMPrgm("
_:
	
	.dw	h1PropZInt
_:	.db	+_--_-1
	.db	"RunPrgm("
_:
	
	.dw	h2PropZInt
_:	.db	+_--_-1
	.db	"RunArchdPrgm("
_:

	.dw	h2SampZInt
_:	.db	+_--_-1
	.db	"AppOnErr("
_:

	.dw	h2SampTInt
_:	.db	+_--_-1
	.db	"AppOffErr"
_:

	.dw	h2SampFTest
_:	.db	+_--_-1
	.db	"GET_ERR#()"
_: